#include <qapplication.h>

#include "cmeter.h"

#include <qtimer.h>
#include <qprocess.h>
#include <qdir.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qdatetime.h>
#include <qmessagebox.h>

#include <qcheckbox.h>
#include <qlineedit.h>
#include "frmsettings.h"

QTimer timerNormal, timerDaemon, timerHide;

CMeter data;

int last_up, last_down;

bool hideImmediately = false, closeImmediately = false;

void frmMain::init()
{
	connect(&timerNormal, SIGNAL(timeout()), SLOT(OnTimer()));
	connect(&timerDaemon, SIGNAL(timeout()), SLOT(OnTimer()));
	connect(&timerHide, SIGNAL(timeout()), SLOT(OnHideImmediately()));

	lblAboutQt->setText(
		"<p>This program uses <b>Qt</b> version <b>" + (QString)QT_VERSION_STR + "</b></p>"
	    	"<p>Qt is a C++ toolkit for multiplatform GUI &amp; "
	    	"application development.</p>"
	    	"<p>Qt provides single-source "
	    	"portability across MS&nbsp;Windows, Mac&nbsp;OS&nbsp;X, "
	    	"Linux, and all major commercial Unix variants."
	    	"<br>Qt is also available for embedded devices.</p>"
	    	"<p>Qt is a Trolltech product. "
	    	"See <tt>http://www.trolltech.com/qt/</tt> "
	    	"for more information.</p>");

	last_up = -1;
	last_down = -1;

	readData();
		
	QDir dir(QDir::homeDirPath() + "/.NeoUpDownMeter");
	QFile file(dir.path() + "/lock");

	if (!dir.exists())
		QDir().mkdir(dir.path());

	if (!file.exists())
	{
		file.open(IO_WriteOnly);
		
		QTextStream stream(&file);
		stream << "DO NOT REMOVE THAT FILE!!\n";

		file.close();

		if (qstrcmp(qApp->argv()[1], "-d") == 0)
		{
			hideImmediately = true;
			timerHide.start(0, true);
			timerDaemon.start(300000);
		}
		else
			timerNormal.start(1000);
	}
	else
	{		
		closeImmediately = true;
		
		QMessageBox msg("Warning", "Another instance of that app is already running!\nNothing will be saved to data file!",
				QMessageBox::Warning, QMessageBox::Ok, QMessageBox::NoButton, QMessageBox::NoButton);
		msg.exec();

		if (qstrcmp(qApp->argv()[1], "-d") == 0)
			exit(0);
	
		timerNormal.start(1000);
	}

	OnTimer();
}

void frmMain::destroy()
{
	if (!closeImmediately)
	{
		writeData();

		QDir dir(QDir::homeDirPath() + "/.NeoUpDownMeter");
		QFile file(dir.path() + "/lock");

		if (file.exists())
			file.remove();
	}	
}

void frmMain::OnTimer()
{
	QProcess * ifconfig = new QProcess;

	ifconfig->addArgument("ifconfig");
	ifconfig->addArgument(data.getInterface());
	
	ifconfig->start();

	while (ifconfig->isRunning());

	QString stdOut(ifconfig->readStdout());

	if (!stdOut.isEmpty())
	{
		//[total values]
		//download
		unsigned int rxBytes_pos = stdOut.find("RX bytes:");
		unsigned int rxBytes_space = stdOut.find(" ", (rxBytes_pos + 9));
		
		QString rxBytes(stdOut.mid((rxBytes_pos + 9), (rxBytes_space - rxBytes_pos - 9)));
			
		lblCurrentTotalDown->setText("<b>Total:</b> " + getStringValue(rxBytes.toUInt() / 1024, 1));
		
		//upload
		unsigned int txBytes_pos = stdOut.find("TX bytes:");
		unsigned int txBytes_space = stdOut.find(" ", (txBytes_pos + 9));
		
		QString txBytes(stdOut.mid((txBytes_pos + 9), (txBytes_space - txBytes_pos - 9)));
			
		lblCurrentTotalUp->setText("<b>Total:</b> " + getStringValue(txBytes.toUInt() / 1024, 1));

		//[current values]
		//download
		if (last_down != -1)
		{
			lblCurrentDown->setText("<b>Current:</b> " + getStringValue((rxBytes.toUInt() - last_down)) + "/s");
		}
		
		last_down = rxBytes.toInt();

		//upload
		if (last_up != -1)
		{
			lblCurrentUp->setText("<b>Current:</b> " + getStringValue((txBytes.toUInt() - last_up)) + "/s");
		}
		
		last_up = txBytes.toInt();

		//average
		QProcess * uptime = new QProcess;

		uptime->addArgument("cat");
		uptime->addArgument("/proc/uptime");

		uptime->start();

		while (uptime->isRunning());

		QString * suptime = new QString(uptime->readStdout());
		(*suptime) = suptime->left(suptime->find("."));

		lblCurrentAverageDown->setText("<b>Average:</b> " + getStringValue(rxBytes.toUInt() / suptime->toUInt()) + "/s");
		lblCurrentAverageUp->setText("<b>Average:</b> " + getStringValue(txBytes.toUInt() / suptime->toUInt()) + "/s");

		delete suptime;
		delete uptime;
	
		//still the same month??
		if (data.getMonth() != QDate::currentDate().month())
		{
			data.setDown(0);
			data.setTotalDown(0);
			data.setUp(0);
			data.setTotalUp(0);
			
			data.setMonth(QDate::currentDate().month());
		}	

		int ip_start = (stdOut.find("inet addr:") + 10);
		int ip_len = (stdOut.find(" ", ip_start) - ip_start );
		QString current_ip(stdOut.mid(ip_start, ip_len));
		
		grpIp->setTitle("IP:" + current_ip);

		if (data.getIp().isEmpty())
			data.setIp(current_ip);
		
		if (data.getIp() == current_ip)
		{	
			data.setDown(rxBytes.toUInt() / 1024);
			data.setUp(txBytes.toUInt() / 1024);
		}
		else
		{
			data.setTotalDown( (data.getTotalDown() + data.getDown()) );
			data.setTotalUp( (data.getTotalUp() + data.getUp()) );
			
			data.setIp(current_ip);
		}

		grpMonth->setTitle("Month:" + QDate::longMonthName(QDate::currentDate().month()));

		lblTotalDown->setText("<b>Total: </b>" + getStringValue(data.getTotalDown() + data.getDown(), 1));
		lblTotalAverageDown->setText
			("<b>Average: </b>" + getStringValue((data.getTotalDown() + data.getDown()) / QDate::currentDate().day(), 1) + "/d");

		lblTotalUp->setText("<b>Total: </b>" + getStringValue(data.getTotalUp() + data.getUp(), 1));
		lblTotalAverageUp->setText
			("<b>Average: </b>" + getStringValue((data.getTotalUp() + data.getUp()) / QDate::currentDate().day(), 1) + "/d");
		
		writeData();
	}
	else
	{
		timerNormal.stop();
		timerDaemon.stop();

		frmSettings * settings = new frmSettings;
		settings->txtInterface->setText(data.getInterface());
		settings->exec();
		
		data.setInterface(settings->txtInterface->text());
		
		delete settings;
		
		writeData();

		if (hideImmediately)
		{
			timerHide.start(0, true);
			timerDaemon.start(300000);
		}
		else
			timerNormal.start(1000);
	}

	delete ifconfig;
}

QString frmMain::getStringValue(double value, unsigned int unit)
{
	/*
	units:
		0 - B
		1 - KiB
		2 - MiB
		3 - GiB
		4 - TiB
	*/

	QString stringValue("");
	unsigned int count = unit;
	double num = value;

	while (num >= 1024)
	{
		num /= 1024;
		count++;
	}
	
	stringValue = (QString::number(num) + " ");
	
	switch (count)
	{
		case 0:
			stringValue += "B";
		break;
		
		case 1:
			stringValue += "KiB";
		break;

		case 2:
			stringValue += "MiB";
		break;

		case 3:
			stringValue += "GiB";
		break;

		case 4:
			stringValue += "TiB";
		break;		
	}

	return stringValue;
}

void frmMain::writeData()
{
	if (!closeImmediately)
	{
		QDir dir(QDir::homeDirPath() + "/.NeoUpDownMeter");
			
		if (!dir.exists())
			QDir().mkdir(dir.path());

		QFile fout(dir.path() + "/data");
		fout.open(IO_WriteOnly);
	
		QTextStream streamout(&fout);
		streamout <<
			QString::number(data.getTotalDown()) << "\n" <<
			QString::number(data.getTotalUp()) << "\n" <<
			data.getIp() << "\n" <<
			QString::number(data.getDown()) << "\n" <<
			QString::number(data.getUp()) << "\n" <<
			QString::number(data.getMonth()) << "\n" <<
			data.getInterface() << "\n";
		fout.close();
	}
}

void frmMain::readData()
{
	QDir dir(QDir::homeDirPath() + "/.NeoUpDownMeter");
	QFile fin(dir.path() + "/data");
			
	if (!dir.exists() || !fin.exists())
	{
		data.setTotalDown(0);
		data.setTotalUp(0);
		
		data.setIp("");
		data.setDown(0);
		data.setUp(0);
		data.setMonth(QDate::currentDate().month());

		data.setInterface("");	

		writeData();
	}
	else
	{	
		fin.open(IO_ReadOnly);
		
		QTextStream streamin(&fin);
		
		data.setTotalDown(streamin.readLine().toDouble());
		data.setTotalUp(streamin.readLine().toDouble());
		data.setIp(streamin.readLine());
		data.setDown(streamin.readLine().toDouble());
		data.setUp(streamin.readLine().toDouble());
		data.setMonth(streamin.readLine().toInt());
		data.setInterface(streamin.readLine());
		
		fin.close();	
	}
}

void frmMain::OnHideImmediately()
{	
	hide();
}
